@@ -209,7 +209,7 @@ def mpp2ppu(mpp: float, units: Union[str, int]) -> float:
     mpp_to_upp = {
         "um": 1,
         "mm": 1e3,
-        "cm": 1e6,
+        "cm": 1e4,
         "inch": 25400,
         2: 25400,
         3: 1e6,
@@ -407,7 +407,7 @@ def mean_pool(image: np.ndarray, pool_size: int) -> np.ndarray:
 
     .. math::
         \left\lfloor
-        \frac{\texttt{image.shape}}{\texttt{pool_size}}
+        \frac{\texttt{image.shape}}{\texttt{pool\_size}}
         \right\rfloor
 
     Args:
@@ -441,10 +441,11 @@ def resize_array(
 
     Tries to use the fastest method available by trying several
     libraries in turn. The order of preference is:
-        1. `cv2.resize`
-        2. 'PIL.Image.resize'
-        3. `scipy.ndimage.zoom`
-        4. Nearest neighbour subsampling
+
+    1. `cv2.resize`
+    2. 'PIL.Image.resize'
+    3. `scipy.ndimage.zoom`
+    4. Nearest neighbour subsampling
 
     Args:
         array (np.ndarray):
@@ -482,8 +483,9 @@ def resize_array(
         }
         cv2_interpolation = str_to_cv2_interpolation[interpolation]
 
+        out_size = tuple(int(x) for x in shape[::-1])
         return cv2.resize(
-            array, shape[::-1], interpolation=cv2_interpolation, **(cv2_kwargs or {})
+            array, out_size, interpolation=cv2_interpolation, **(cv2_kwargs or {})
         )
 
     with suppress(ImportError):
